<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00004000ContratoService as ContratoService;
use Teknisa\SM\Service\Sm00004000DocumentoContratoService as DocumentoContratoService;
use Teknisa\SM\Service\Sm00005000ParametrizacoesServicesService;
use Teknisa\SM\Service\Sm00015004AgendamentoDeRecursoService;
use Teknisa\SM\Service\Sm00001000LancamentoPropostaService;
use Teknisa\SM\Controller\Sm00001000LancamentoPropostaController;
use Teknisa\SM\Service\Sm00004000ItemContratoService;
use Teknisa\Libs\Util\NewCode;

class Sm00004000ContratoController extends SMCrud
{
    protected $dataSourceName = 'fct_contrato';
    //protected $dataSourceNameToSave = 'fct_contrato';
    protected $contratoService;
    protected $documentoService;
    protected $parametrogeral;
    protected $agedRecPendente;
    protected $propostaService;
    protected $propostaController;
    protected $newCode;
    protected $itemContratoService;
    protected $codigoEscopoFaturamentoContratoGerado;

    public function __construct($crudService, $environment, ContratoService $contratoService, DocumentoContratoService $documentoService, Sm00005000ParametrizacoesServicesService $parametrogeral, Sm00015004AgendamentoDeRecursoService $agedRecPendente, Sm00001000LancamentoPropostaService $propostaService, Sm00001000LancamentoPropostaController $propostaController, NewCode $newCode, Sm00004000ItemContratoService $itemService, $codigoEscopoGerado) {
        parent::__construct($crudService, $environment);
        $this->contratoService    = $contratoService;
        $this->documentoService   = $documentoService;
        $this->parametrogeral     = $parametrogeral;
        $this->agedRecPendente    = $agedRecPendente;
        $this->propostaService    = $propostaService;
        $this->propostaController = $propostaController;
        $this->newCode            = $newCode;
        $this->itemContratoService= $itemService;
        $this->codigoEscopoFaturamentoContratoGerado = $codigoEscopoGerado;
    }
    
    /**
     * Busca os dados do contrato adicionando o filtro por 
     * operador responsavel para a hierarquia de operadores
    **/
    /*public function find(Request\Filter $request, Response $response){
        try{
            $nrorg = $this->environment->getNrOrg();
            $idUtilizaOper = $this->parametrogeral->getIdUtilizaHierarquiaOper($nrorg);
            if($idUtilizaOper == 'N')
                $this->dataSourceName = 'fct_contrato';
            else 
                $request->getFilterCriteria()->addCondition("CDOPERADORLOGADO", "=", $this->environment->getCdOperador());
            
            parent::find($request, $response);  
            $this->dataSourceName = 'fct_contrato_widget';
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }*/
    
    /**
     * A função "copiaVersionaContrato" executa a procedure
     * "FCT_COPIA_VERSIONA_CONTRATO".
     */
    public function copiaVersionaContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqcontrato   = $row['NRSEQCONTRATO']; 
            $idCopiaVersiona = $row['TIPO'];
     
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $adcAgrupamento = isset($row["ADCAGRUPAMENTO"]) ? $row["ADCAGRUPAMENTO"] : false;
            $idParcelaConsiderada = isset($row["IDPARCELACONSIDERADA"]) ? $row["IDPARCELACONSIDERADA"] : null;
            $idReplicarNegociacao = "N";
            $dtRenegociaCont = isset($row["DTRENEGOCIACONT"]) ? $row["DTRENEGOCIACONT"] : null;
            $dtFimContrato  = isset($row["DTFIMCONTRATO"]) ? $row["DTFIMCONTRATO"] : null;
            
            $contratoGerado = $this->contratoService->executarProcedureFctCopiaVersionaContrato($nrseqcontrato, $idCopiaVersiona, $operador, $nrorg, $adcAgrupamento, $idParcelaConsiderada, $idReplicarNegociacao, $dtRenegociaCont,$dtFimContrato);
            if($idCopiaVersiona == 'V'){
                
                $this->contratoService->excluirDocumentoCertsign($nrseqcontrato, $nrorg, $operador);
            }
            $this->contratoService->anexarDocumentosContratosCopiados($contratoGerado, $nrorg);
            $this->contratoService->salvarAnexoRenegociacaoContrato($nrseqcontrato, $nrorg, $operador,$contratoGerado);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * A função "encerraContrato" executa a procedure
     * "FCT_ENCERRA_CONTRATO".
     */
    public function encerraContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrseqcontrato       = $row['NRSEQCONTRATO']; 
            $idsituacaocontrato  = $row['IDSITUACAOCONTRATO']; 
            $dtrecisao           = isset($row["DTRECISAOCONTR"]) ? $row['DTRECISAOCONTR'] : null; 
            $idtpexclusaoparcela = isset($row["IDTPEXCLUSAOPARCELA"]) ? $row['IDTPEXCLUSAOPARCELA'] : 2; 
            $nrtiporescisao      = $row["NRTIPORESCISAO"];
            $idDocX              = $row["IDDOCX"];
            
            $dtpedrescisaocontr = $row["DTPEDRESCISAOCONTR"];
            $nrmotivorescisao   = $row["NRMOTIVORESCISAO"];
            $dsjustificaencerra = $row["DSJUSTIFICAENCERRA"];
            $vrpendenciafin     = $row["VRPENDENCIAFIN"];
            
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            
            $this->contratoService->encerraContrato($nrseqcontrato, $idsituacaocontrato, $dtrecisao, $idtpexclusaoparcela, $nrorg, $operador, $nrtiporescisao, $idDocX, $dtpedrescisaocontr, $nrmotivorescisao, $dsjustificaencerra, $vrpendenciafin);

            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "geraParcelasContrato" executa a procedure
     * "FCT_VENCPREV_CONT".
     */
    public function geraParcelasContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqcontrato = $row['NRSEQCONTRATO'];
            $nrorg = $this->environment->getNrOrg();
            
            $this->contratoService->executarProcedureFctVencPrevCont($nrseqcontrato, $nrorg);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "geraParcelasContrato" executa a procedure
     * "FCT_GERAR_PARCELAS_FATURA".
     */
    public function geraParcelasContratoNovoFat(Request\Row $request, Response $response) {
        try {
            $nrorg    = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            
            $row = $request->getRow();
            
            $nrseqcontrato = $row['NRSEQCONTRATO'];
            $nrParc = (isset($row['NRPARCNEGOCIO'])) ? $row['NRPARCNEGOCIO'] : null;
            $idUtilizaComp = (isset($row['IDUTILIZACOMP'])) ? $row['IDUTILIZACOMP'] : "N";
            $idReplicarNegociacao = isset($row["IDREPLICARNEGOCIACAO"]) ? $row["IDREPLICARNEGOCIACAO"] : "N";
            
            $result = $this->contratoService->gerarParcelasContratoNovoFat($nrseqcontrato, $idUtilizaComp, $nrorg, $operador, $nrParc, $idReplicarNegociacao, $row); 
            //continuacao do service para evitar dependência circular
            $r = $this->finalizacaoDeContratosGeradosAutomaticamente($result);
            $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "cancelaParcelasContrato" executa a procedure
     * "FCT_EXCLUIR_PARCELAS_FATURA".
     */
    public function cancelaParcelasContrato (Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
                
            $nrseqcontrato = $row['NRSEQCONTRATO'];
            $nrorg = $this->environment->getNrOrg();
             
            $this->contratoService->executarProcedureFctExcluirParcelasFatura($nrseqcontrato, $nrorg);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "recebeParamFatModCont" recebe os dados de Faturamento do Modelo de Contrato
     */
    public function recebeParamFatModCont(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $seqModContrato = $row['NRSEQMODCONTRA'];
            $nrorg = $this->environment->getNrOrg();
            
            $ValidaParamFatModCont = $this->contratoService->recebeParamFatModCont($seqModContrato, $nrorg);
            
            $dtsValParamFatModCont = new DataSet("PARAMFATMODCONT", $ValidaParamFatModCont);
            
            $response->addDataSet($dtsValParamFatModCont);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    /**
     * A função "recebeModReajustContr" recebe os dados de Reajuste do Modelo de Contrato
     */
    public function recebeModReajustContr(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $seqModContrato = $row['NRSEQMODCONTRA'];
            $nrorg = $this->environment->getNrOrg();
            
            $ValidaModReajustContr = $this->contratoService->recebeModReajustContr($seqModContrato, $nrorg);
            
            $dtsValModReajustContr = new DataSet("MODREAJUSTCONTR", $ValidaModReajustContr);
            
            $response->addDataSet($dtsValModReajustContr);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    /**
     * A função "recebeModIndContr" recebe os dados de Indice de Reajuste do Modelo de Contrato
     */
    public function recebeModIndContr(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $seqModContrato = $row['NRSEQMODCONTRA'];
            $seqModReajus = $row['NRSEQMODREAJUS'];
            $nrorg = $this->environment->getNrOrg();
            
            $ValidaModIndContr = $this->contratoService->recebeModIndContr($seqModContrato, $seqModReajus, $nrorg);
            
            $dtsValModIndContr = new DataSet("MODINDCONTR", $ValidaModIndContr);
            
            $response->addDataSet($dtsValModIndContr);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    /**
     * A função "recebeModeloDocContr" recebe os dados de Documento do Modelo de Contrato
     */
    public function recebeModeloDocContr(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $seqModContrato = $row['NRSEQMODCONTRA'];
            $nrorg = $this->environment->getNrOrg();
            
            $ValidaModeloDocContr = $this->contratoService->recebeModeloDocContr($seqModContrato, $nrorg);
            
            $dtsModeloDocContr = new DataSet("MODELODOCCONTR", $ValidaModeloDocContr);
            
            $response->addDataSet($dtsModeloDocContr);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    /**
     * A função "geraCodigoContrato" executa a procedure "CRIA_MNEMONICO".
     */
    public function geraCodigoContrato(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
             
            $seqContrato = $row['NRSEQCONTRATO'];
            $nrorg = $this->environment->getNrOrg();
          
            $codigoContr = $this->contratoService->executarProcedureCriaMnemonico($seqContrato, $nrorg);
          
            $dtsCodigoContr = new DataSet("CODIGOCONTR", array("CODIGO" => $codigoContr));
             
            $response->addDataSet($dtsCodigoContr);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function gerarDocumentoContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrorg         = $row["NRORG"];
            $idSituacao    = $row['IDSITUACAOCONTRATO'];
            $idExtensao    = $row["IDPERMITEDOCX"];
            
            $caminhoArquivos = $this->documentoService->gerarDocumentoContrato($nrseqcontrato, $nrorg, $idSituacao, $idExtensao);
           
            if (count($caminhoArquivos) > 0) {
                $messageerror = null;
                $caminhosFisicos = array();
                foreach ($caminhoArquivos as $chave => $arq) {
                    if(!$arq["CAMINHO"])
                        $messageerror = "Atenção!<br><br>Não é possível gerar o documento deste contrato pois não há itens ligados a ele.";
                        
                    $retorno[] = $arq["CAMINHO"];
                }
                
                $response->addMessage(new Response\Message($messageerror));
                if (count($retorno) > 0) 
                    $response->addDataSet(new DataSet("documentos", array("links" => $retorno)));
                else 
                    throw new \Exception("Falha ao anexar o(s) documento(s) no FileServer.");
            } else {
                throw new \Exception("Falha ao gerar o(s) documento(s).");
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * salva a alteração dos operadores responsáveis dos contratos selecionados
    **/
    public function salvaAlteracaoOperContrato(Request\Row $request, Response $response){
        try{
            $rows          = $request->getRow();
            $nrseqcontrato = $rows["NRSEQCONTRATO"];
            $operador      = $rows["CDOPERRESPONSAVEL"];
            $nrorg         = $this->environment->getNrOrg();
            $nrseqcontrato = implode(",", $nrseqcontrato);
            $sql           = "NRSEQCONTRATO IN($nrseqcontrato)";
            $this->contratoService->atualizaOperadorResponsavel($sql, $operador, $nrorg);
            if($rows["ALTERAPROPOSTA"] == 'Y'){
                $sql = "NRSEQORCAMENTO IN(SELECT NRSEQORCAMENTO FROM FCT_REFEORCACONT WHERE NRSEQCONTRATO IN($nrseqcontrato))";
                $this->propostaService->atualizaOperadorResponsavel($sql, $operador, $nrorg);
            }
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function atualizaAgendamentosPend(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrseqcontrato = $row['NRSEQCONTRATO'];
            $nrorg         = $this->environment->getNrOrg();
            
            $this->agedRecPendente->atualizarAgendamentoPendente('1', null, $nrseqcontrato, null, null, $nrorg);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Retorna o parametro validação para a geração de 
     * documentos em pdf
    **/
    public function getParametroExtensaoDoc(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrorg         = $this->environment->getNrOrg();
            
            $param = $this->contratoService->getParametroExtensaoDoc($nrseqcontrato, $nrorg);
            $response->addDataSet(new DataSet("IDPERMITEDOCX", $param));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * A função "geraDocumentoContrato" executa a procedure "FCT_SUB_MNE_DOC".
     */
    /*public function geraDocumentoContrato(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
         
            $seqContrato = $row['NRSEQCONTRATO'];
            $nrSeqDocCont = $row['NRSEQCONTDOC'];
  
            $nrorg = $this->environment->getNrOrg();
      
            $this->contratoService->executarProcedureFctSubMneDoc($seqContrato, $nrSeqDocCont, $nrorg);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }*/
    
  /**
    * Retorna os dados de faturamento do modelo de contrato
    */
    public function getParamVencimentos(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $nrseqmodcontra = $row["NRSEQMODCONTRA"];
            $nrorg          = $this->environment->getNrOrg();
            
            $param = $this->contratoService->getParamVencimentos($nrseqmodcontra, $nrorg);
            $response->addDataSet(new DataSet("PARAM", $param));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
      * Retorna os produto de faturamento do serviço
     */
    public function getProdServico(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $cdServOrca = $row["CDSERVORCA"];
            $nrorg      = $this->environment->getNrOrg();
            
            $param = $this->contratoService->getProdServico($cdServOrca, $nrorg);
            $response->addDataSet(new DataSet("SERV", $param));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Passa o request do pedido de uma parcela e retorna os valores via Response.
    */
    public function getDadosUmaParcela(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrFaturaCont = $row["NRFATURACONT"];
            $param = $this->contratoService->getDadosUmaParcela($nrFaturaCont);
            $response->addDataSet(new DataSet("PARCELA", $param));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
      * Atualiza o valor da parcela de Faturamento ao alterar os itens
     */
    public function AtulizaValorFaturamento(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $nrFaturaCont   = $row["NRFATURACONT"];
            $nrSeqContrato  = $row["NRSEQCONTRATO"];
            $nrorg          = $this->environment->getNrOrg();
            $operador       = $this->environment->getCdOperador();
            
            $this->contratoService->AtulizaValorFaturamento($nrFaturaCont, $nrSeqContrato, $nrorg, $operador);
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Atualiza na tabela de documento do contrato o documento
     * de acordo com a situação
    **/
    public function atualizaDocumentoSituacao(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $idsituacao    = $row["IDSITUACAOCONTRATO"];
            $nrorg         = $this->environment->getNrOrg();
            $this->contratoService->atualizaDocumentoSituacao($nrseqcontrato, $nrorg, $idsituacao);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaIntervaloDatas(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $dtInicio = $row["DTINICIO"];
            $dtFinal  = $row["DTFINAL"];
            
            $dtInicio = substr($dtInicio, 0, 10);
            $dtFinal = substr($dtFinal, 0, 10);
            
            $intervalo = $this->contratoService->retornaIntervaloDatas($dtInicio, $dtFinal);
            $response->addDataSet(new DataSet("DATA", $intervalo));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "buscaFilialHierarquia" busca a filial vinculado ao operador 
     * na hierarquiia de operadores (CRM_HIERARQUIAOPER)
     */
    public function buscaFilialHierarquia(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $operador = $row["OPERADOR"];
            $nrorg    = $this->environment->getNrOrg();
            
            $filial = $this->contratoService->buscaFilialHierarquia($operador, $nrorg);
            $response->addDataSet(new DataSet("DATA", $filial));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /* Retorna os dados de faturamento da parametrização  de contrato
     */
    public function getParamFatCont(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $nrseqcontrato  = $row["NRSEQCONTRATO"];
            $nrorg          = $this->environment->getNrOrg();
            
            $param = $this->contratoService->getParamFatCont($nrseqcontrato, $nrorg);
            $response->addDataSet(new DataSet("PARAM", $param));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /* Valida se o orçamento foi realizado através de Composição de Serviços.
     */
    public function validaCompServico(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $nrseqcontrato = $row['NRSEQORCAMENTO']; 
        $nrorg          = $row['NRORG'];
        
        $compServico = $this->contratoService->validaCompServico($nrseqcontrato, $nrorg);
        
        $response->addDataSet(new DataSet("COMPSERVICO", $compServico));
    }
    /**
     * Insere uma proposta com composição de serviço a partir de um contrato
     * (Utilizada apenas na inserção de contratos com composição de serviço e medidas)
    **/
    public function salvarContratoCompMedida(Request\Row $request, Response $response){
        try {
            $row            = $request->getRow();
            $nrseqorcamento = null;
            
            $userId     = $request->getUserId();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            //Busca e salva as informações da proposta
            $infoOrca = $this->contratoService->getInfoModeloPropostaInserindo($row["CONTRATO"], $nrorg);
            $requestProp  = new Request\DataSet(new DataSet("", array(new Row($infoOrca))), "product#/fct_orcamento_widget/save", "POST", $userId);
            $responseProp = new Response();
            $this->propostaController->save($requestProp, $responseProp);
            
            if($responseProp->getError())
                throw new \Exception($responseProp->getError()->getMessage());
            
            //Insere os elementos, medidas e itens
            $nrseqorcamento    = $responseProp->getDataSets()[0]->getRows()[0]["NRSEQORCAMENTO"];
            $escopos           = $row["CONTRATO"]["NRTIPOORCAMENTO"];
            $nrorcamentomodelo = $row["CONTRATO"]["NRORCAMENTOMODELO"];
            
            $this->propostaService->inserirTudo($nrseqorcamento, $escopos, $nrorcamentomodelo, $row["MEDIDAS"], $row["ITENS"]);

            //guarda as informações da tabela de preço e do serviço
            $this->propostaService->salvarInfoServicoEPrecoOrca($nrseqorcamento, $escopos, $nrorg, $cdoperador);
            // aceita a proposta gerada
            $this->propostaService->aceitarProposta($nrseqorcamento, $nrorg, $row["CONTRATO"]["NRPARCNEGOCIO"], $cdoperador);
            //Altera as informações de referencia, datas e observação do contrato
            $nrcontrato = $this->contratoService->alterarInformacoesContrato($row["CONTRATO"], $nrseqorcamento, $nrorg);
            
            if($nrcontrato) {
                $columnsToReturn = array(
                    "NRSEQCONTRATO" => $nrcontrato,
                    "NRORG"         => $nrorg
                );
                $response->addDataSet(new DataSet("columnsToReturn", array($columnsToReturn)));
            }
            
        } catch(\Exception $e){
            if($nrseqorcamento)
                $this->propostaService->excluirProposta($nrseqorcamento, $nrorg);
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function parcelarNegociacaoPorIgual(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            $nrnegociacaoproposta = $row["NRNEGOCIACAOPROPOSTA"];
            $qtparcelas           = $row["QTPARCELAS"];
            $qtdiasintervalo      = $row["QTDIASINTERVALO"];
            $nrdiavencimento      = $row["NRDIAVENCIMENTO"];
            $qtcarenciadias       = $row["QTCARENCIADIAS"];
            $vrtotitemcomdesc     = $row["VRTOTITEMCOMDESC"];
            $idAlteraProposta     = isset($row["IDREPLICARORCA"]) ? $row["IDREPLICARORCA"] : false;

            $this->contratoService->parcelarNegociacaoPorIgual($nrnegociacaocontrato, $nrnegociacaoproposta, $qtparcelas, $qtdiasintervalo, $nrdiavencimento, $qtcarenciadias, $vrtotitemcomdesc, $idAlteraProposta);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validarNegociacaoValores(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            
            $resultado = $this->contratoService->validarNegociacaoValores($nrseqcontrato);
            $response->addDataSet(new DataSet("validacao", array("resultado" => $resultado)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /*public function replicarDescontoItensNegociacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            $vrdescitem           = $row["VRDESCITEM"];
            
            $this->contratoService->replicarDescontoItensNegociacao($nrnegociacaocontrato, $vrdescitem);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }*/

    public function atualizarValoresNegociacaoAoEditarElemento(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato   = $row["NRNEGOCIACAOCONTRATO"];
            $nritemcontratopai      = $row["NRITEMCONTRATOPAI"];
            
            $valorNegociacao = $this->contratoService->atualizarValoresNegociacaoAoEditarElemento($nrnegociacaocontrato, $nritemcontratopai);
            $response->addDataSet(new DataSet("retorno", array("valorNegociacao" => $valorNegociacao)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function desagruparItemNegociacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $negociacao       = $row["NEGOCIACAO"];
            $itemNegociacao   = $row["ITEM"];
            $nrseqcontrato    = $row["NRSEQORCAMENTO"];
            $idGeradoPor      = $row["IDGERADOPOR"];
            $nrseqcontratopai = $row["NRSEQCONTRATOPAI"];
            
            $valorNegociacao = $this->contratoService->desagruparItemNegociacao($nrseqcontrato, $negociacao, $itemNegociacao, $idGeradoPor, $nrseqcontratopai);
            $response->addDataSet(new DataSet("retorno", array("valorNegociacao" => $valorNegociacao)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getQuantidadeItensNegociacaoContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            $nrorg = $this->environment->getNrOrg();

            $quantidade = $this->contratoService->getQuantidadeItensNegociacaoContrato($nrnegociacaocontrato, $nrorg);
            $response->addDataSet(new DataSet("retorno", array("quantidade" => $quantidade)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvarDataFinalValidadeNegociacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato  = $row["NRNEGOCIACAOCONTRATO"];
            $nrdescperiodonegocont = $row["NRDESCPERIODONEGOCONT"];
            $qtmesesdesc           = $row["QTMESESDESC"];
            $dataAtual             = $row["DTINIVIGENCONT"];
            $nrorg = $this->environment->getNrOrg();

            $dataValidade = $this->contratoService->salvarDataFinalValidadeNegociacao($nrnegociacaocontrato, $nrdescperiodonegocont, $nrorg, $dataAtual);
            $response->addDataSet(new DataSet("retorno", array("dataValidade" => $dataValidade)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * aplica as informações dos parametros de faturamento
     * as parcelas do período.
    **/
    public function paralisarFaturamentoContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $dtInicio      = $row["DTINIPARALISACAO"];
            $dtFinal       = $row["DTRETOMADAFATURA"];
            $nrorg         = $this->environment->getNrOrg();
            $cdoperador    = $this->environment->getCdOperador();
            
            $this->contratoService->alterarSituacaoParcelasEMedicoes($nrseqcontrato, $dtInicio, $dtFinal, '5', $nrorg, $cdoperador);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Retoma o faturamento do contrato a partir de determinada data
    **/
    public function retormarFaturamentoContrato(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $dsmotivo      = $row["DSMOTIVORETOMADA"];
            $nrparalisacao = $row["NRPARAMPARALISAO"];
            $nrorg         = $this->environment->getNrOrg();
            $cdoperador    = $this->environment->getCdOperador();
            
            $this->contratoService->retormarFaturamentoContrato($nrseqcontrato, $dsmotivo, $nrparalisacao, $nrorg, $cdoperador);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function atualizarNegociacaoProposta(Request\Row $request, Response $response) {
        try {
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $row = $request->getRow();
            $negociacao = $row["NEGOCIACAO"];
            
            $this->contratoService->atualizarNegociacaoProposta($negociacao);
            $this->contratoService->atualizarValoresDescontoPorPeriodoDaNegociacao($negociacao["VRTOTITEMCOMDESC"], $negociacao["NRNEGOCIACAOCONTRATO"], $nrorg, $cdoperador);
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Função utilizada para dar Update nos itens de uma negociação de proposta
    */
    public function atualizarItemNegociacaoProposta(Request\Row $request, Response $response) {
        try {
            $row                  = $request->getRow();
            $itemNegociacao       = $row["ITNEGOCIACAO"];
            $nrnegociacaoproposta = $row["NRNEGOCIACAOPROPOSTA"];
            $cdoperador           = $this->environment->getCdOperador();
            $nrorg                = $this->environment->getNrOrg();

            $valorNegociacao = $this->contratoService->atualizarItemNegociacaoProposta($nrnegociacaoproposta, $itemNegociacao, $nrorg, $cdoperador);

            $response->addDataSet(new DataSet("retorno", array("valorNegociacao" => $valorNegociacao)));
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function atualizarValoresDescontoPorPeriodoDaNegociacao(Request\Row $request, Response $response) {
    	try {
    		$nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();

    		$row = $request->getRow();
    		$valorNegociacao      = $row["VRTOTITEMCOMDESC"];
    		$nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];

    		$this->contratoService->atualizarValoresDescontoPorPeriodoDaNegociacao($valorNegociacao, $nrnegociacaocontrato, $nrorg, $cdoperador);
    	}
    	catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function atualizarParceiroNegociacoesContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrparcnegocio = $row["NRPARCNEGOCIO"];

            $this->contratoService->atualizarParceiroNegociacoesContrato($nrseqcontrato, $nrparcnegocio);
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Atualiza os dados de um contrato após alterações nos elementos 
     * ou tipos de medida variáveis
    **/
    public function atualizarItensContrato(Request\Row $request, Response $response) {
         try {
            $row          = $request->getRow();
            $nrorg        = $this->environment->getNrOrg();
            $cdoperador   = $this->environment->getCdOperador();
            $contrato     = $row["CONTRATO"];
            $elementos    = $row["ELEMENTOS"];
            $composicoes  = $row["COMPOSICOES"];
            $tptratamento = $row["TPTRATAMENTO"];
            
            $mensagem = $this->contratoService->atualizarItensContrato($contrato, $elementos, $composicoes, $nrorg, $cdoperador, $tptratamento, false);
            $response->addMessage(new Response\Message($mensagem));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Atualiza os itens do contrato e seus valores de acordo com as alterações feitas nos tipos de medida variáveis.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function atualizarItensContratoPeloTiposDeMedida(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $contrato     = $row["CONTRATO"];
            $nrorg        = $this->environment->getNrOrg();
            $cdoperador   = $this->environment->getCdOperador();
            $tptratamento = $row["TPTRATAMENTO"];
            
            $this->contratoService->atualizarTipoDeMedidaContrato($contrato, $nrorg, $cdoperador, $tptratamento);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Atualiza dados da proposta e do contrato em contratos de composição de serviço
    **/
    public function atualizarItensContratoProposta(Request\Row $request, Response $response) {
        try {
            $row           = $request->getRow();
            $nrorg         = $this->environment->getNrOrg();
            $cdoperador    = $this->environment->getCdOperador();

            $this->contratoService->atualizarInformacoesContratoProposta($row, $nrorg, $cdoperador);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Salva as informações do serviço e da variação de preço apos a aprovação de um contrato
    **/
    public function salvarDadosMedidaContrato(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrmoeda  = isset($row["NRMOEDA"]) ? $row["NRMOEDA"] : null;

            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->contratoService->salvarDadosMedidaContrato($nrseqcontrato, $nrorg, $cdoperador, $nrmoeda);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Replica a inserção do desconto por período para as propostas
     * geradas a partir de determinado contrato
    **/
    public function salvarDescontoPorPeriodo(Request\DataSet $request, Response $response) {
        try {
            $row        = $request->getDataSet()->getRows()[0];
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            if($row["IDREPLICARORCA"]) {
                $nrdescperiodonegoprop = (!$row["__is_new"])  ?  $row["NRDESCPERIODONEGOPROP"] : $this->newCode->getCodeByTableName("FCT_DESCPERIODONEGOPROP", $nrorg, 1, 10, true);
                $dataset[] = array(
                    "__is_new"              => $row["__is_new"],
                    "NRDESCPERIODONEGOPROP" => $nrdescperiodonegoprop,
                    "NRNEGOCIACAOPROPOSTA"  => $row["NRNEGOCIACAOPROPOSTA"],
                    "VRPERCDESCITEM"        => $row["VRPERCDESCITEM"],
                    "VRDESCITEM"            => $row["VRDESCITEM"],
                    "VRTOTITEMCOMDESC"      => $row["VRTOTITEMCOMDESC"],
                    "QTMESESDESC"           => $row["QTMESESDESC"],
                    "DTFINALVALIDADEDESC"   => $row["DTFINALVALIDADEDESC"],
                    "NRORDEMAPLICADESC"     => $row["NRORDEMAPLICADESC"],
                    "IDTPDESCONTO"          => $row["IDTPDESCONTO"],
                    "NRORG"                 => $nrorg,
                    "IDATIVO"               => "S"
                );
                
                $responseDescontoProp = new Response();
                $requestDescontoProp  = new Request\DataSet(new DataSet("", $dataset), "product#/fct_descperiodonegoprop_negociacaoproposta/save", "POST", $request->getUserId());
                $this->dataSourceNameToSave = "fct_descperiodonegoprop_negociacaoproposta";
                $requestDescontoProp->setParameter('dbname', 'fct_descperiodonegoprop_negociacaoproposta');
                parent::save($requestDescontoProp, $responseDescontoProp);
                $erro = $responseDescontoProp->getError();
                if ($erro) {
                    throw new \Exception($erro->getMessage(), $erro->getErrorCode());
                }
                $row["NRDESCPERIODONEGOPROP"] = $nrdescperiodonegoprop;
            } 
            
            if($row["__is_new"]) 
                $row["NRDESCPERIODONEGOCONT"] = $this->newCode->getCodeByTableName("FCT_DESCPERIODONEGOCONT", $nrorg, 1, 10, true);
            
            $response            = new Response();
            $requestDescontoCont = new Request\DataSet(new DataSet("", array($row)), "product#/fct_descperiodonegocont_negociacaocontrato/save", "POST", $request->getUserId());
            $this->dataSourceNameToSave = "fct_descperiodonegocont_negociacaocontrato";
            $requestDescontoCont->setParameter('dbname', 'fct_descperiodonegocont_negociacaocontrato');
            parent::save($requestDescontoCont, $response);

            $this->dataSourceNameToSave = "fct_contrato_widget";
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Deleta na proposta e no contrato um registro da tabela de desconto por periodo
    **/
    public function deletarDescontoPorPeriodo(Request\Row $request, Response $response) {
        try {
            $requestDescontoCont = new Request\DataSet(new DataSet("", array($request->getRow())), "product#/fct_descperiodonegocont_negociacaocontrato/remove", "POST", $request->getUserId());
            $this->dataSourceNameToSave = "fct_descperiodonegocont_negociacaocontrato";
            $requestDescontoCont->setParameter('dbname', 'fct_descperiodonegocont_negociacaocontrato');
            parent::delete($requestDescontoCont, $response);
            
            $responseDescontoProp = new Response();
            $requestDescontoProp  = new Request\DataSet(new DataSet("", array($request->getRow())), "product#/fct_descperiodonegoprop_negociacaoproposta/remove", "POST", $request->getUserId());
            $this->dataSourceNameToSave = "fct_descperiodonegoprop_negociacaoproposta";
            $requestDescontoProp->setParameter('dbname', 'fct_descperiodonegoprop_negociacaoproposta');
            parent::delete($requestDescontoProp, $responseDescontoProp);
            
            $this->dataSourceNameToSave = "fct_contrato_widget";
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Replica para a competencia da propostas as alterações realizadas no contrato
    **/
    public function salvarCompetenciaNegociacao(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            if($row["IDREPLICARORCA"]) {
                if($row["__is_new"])
                    $row["NRCOMPETENCIANEGOPROP"] = $this->newCode->getCodeByTableName("FCT_COMPETENCIANEGOPROP", $nrorg, 1, 10, true);
                
                $dataset[] = $row;
                $responseCompProposta = new Response(); 
                $requestCompProposta  = new Request\DataSet(new DataSet("", $dataset), "product#/fct_competencianegoprop/save", "POST", $request->getUserId());
                $this->dataSourceNameToSave = "fct_competencianegoprop";
                $requestCompProposta->setParameter('dbname', 'fct_competencianegoprop');
                parent::save($requestCompProposta, $responseCompProposta);    
            }
            
            if($row["__is_new"])
                $row["NRCOMPETENCIANEGOCONT"] = $this->newCode->getCodeByTableName("FCT_COMPETENCIANEGOCONT", $nrorg, 1, 10, true);
                    
            $requestCompetencia  = new Request\DataSet(new DataSet("", array($row)), "product#/fct_competencianegocont/save", "POST", $request->getUserId());
            $this->dataSourceNameToSave = "fct_competencianegocont";
            $requestCompetencia->setParameter('dbname', 'fct_competencianegocont');
            parent::save($requestCompetencia, $response);    
            
            $this->dataSourceNameToSave = "fct_contrato_widget";
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Deleta na proposta e no contrato um registro da tabela de competencia
    **/
    public function deletarCompetenciaNegociacao(Request\Row $request, Response $response) {
        try {
            $responseCompetenciaCont = new Response();
            $requestCompetenciaCont  = new Request\DataSet(new DataSet("", array($request->getRow())), "product#/fct_competencianegocont/remove", "POST", $request->getUserId());
            $this->dataSourceNameToSave = "fct_competencianegocont";
            $requestCompetenciaCont->setParameter('dbname', 'fct_competencianegocont');
            parent::delete($requestCompetenciaCont, $responseCompetenciaCont);
            
            $requestCompetenciaProp = new Request\DataSet(new DataSet("", array($request->getRow())), "product#/fct_competencianegoprop/remove", "POST", $request->getUserId());
            $this->dataSourceNameToSave = "fct_competencianegoprop";
            $requestCompetenciaProp->setParameter('dbname', 'fct_competencianegoprop');
            parent::delete($requestCompetenciaProp, $response);
            
            $this->dataSourceNameToSave = "fct_contrato_widget";
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Replica para as parcelas da propostas as alterações realizadas no contrato
    **/
    public function salvarParcelasNegociacao(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            if($row["IDREPLICARORCA"]) {
                if($row["__is_new"])
                    $row["NRPARCELANEGOPROP"] = $this->newCode->getCodeByTableName("FCT_PARCELANEGOPROP", $nrorg, 1, 10, true);
                
                $dataset[] = $row;
                $responseParcProposta = new Response(); 
                $requestParcProposta  = new Request\DataSet(new DataSet("", $dataset), "product#/fct_parcelanegoprop/save", "POST", $request->getUserId());
                $this->dataSourceNameToSave = "fct_parcelanegoprop";
                $requestParcProposta->setParameter('dbname', 'fct_parcelanegoprop');
                parent::save($requestParcProposta, $responseParcProposta);    
            }
            
            if($row["__is_new"])
                $row["NRPARCELANEGOCONT"] = $this->newCode->getCodeByTableName("FCT_PARCELANEGOCONT", $nrorg, 1, 10, true);
                    
            $requestParcela  = new Request\DataSet(new DataSet("", array($row)), "product#/fct_parcelanegocont/save", "POST", $request->getUserId());
            $this->dataSourceNameToSave = "fct_parcelanegocont";
            $requestParcela->setParameter('dbname', 'fct_parcelanegocont');
            parent::save($requestParcela, $response);    
            
            $this->dataSourceNameToSave = "fct_contrato_widget";
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Deleta na proposta e no contrato um registro da tabela de parcelas
    **/
    public function deletarParcelasNegociacao(Request\Row $request, Response $response) {
        try {
            $responseParcelaCont = new Response();
            $requestParcelaCont  = new Request\DataSet(new DataSet("", array($request->getRow())), "product#/fct_parcelanegocont/remove", "POST", $request->getUserId());
            $this->dataSourceNameToSave = "fct_parcelanegocont";
            $requestParcelaCont->setParameter('dbname', 'fct_parcelanegocont');
            parent::delete($requestParcelaCont, $responseParcelaCont);
            
            $requestParcelaProp = new Request\DataSet(new DataSet("", array($request->getRow())), "product#/fct_parcelanegoprop/remove", "POST", $request->getUserId());
            $this->dataSourceNameToSave = "fct_parcelanegoprop";
            $requestParcelaProp->setParameter('dbname', 'fct_parcelanegoprop');
            parent::delete($requestParcelaProp, $response);
            
            $this->dataSourceNameToSave = "fct_contrato_widget";
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    /**
     * Valida a quantidade máxima de parcelas em uma negociação
    **/
    public function validarQuantidadeMaxParcelas(Request\Row $request, Response $response) {
        try {
            $row            = $request->getRow();
            $nrorg          = $this->environment->getNrOrg();
            $cdoperador     = $this->environment->getCdOperador();
            
            $idTpValidacao        = $row["IDTPVALIDACAO"];
            $nrtipoorcamento      = $row["NRTIPOORCAMENTO"];
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            
            $nrmaxparcelas   = $this->contratoService->validarQuantidadeMaxParcelas($nrtipoorcamento, $nrnegociacaocontrato, $cdoperador, $nrorg);
            
            if($idTpValidacao == "PARCELAUNICA") 
                $nrmaxparcelas = $this->contratoService->validarQuantidadeParcelasInseridas($nrnegociacaocontrato, $nrmaxparcelas, $nrorg);
            
            $response->addDataSet(new DataSet("fct_contrato_widget", array("NRMAXPARCELAS" => $nrmaxparcelas)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function atualizarValorTodasNegociacoesComSomaDosItens(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];

            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();

            $this->contratoService->atualizarValorTodasNegociacoesComSomaDosItens($nrseqcontrato, $nrorg, $cdoperador);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function atualizarValorNegociacaoComSomaDosItens(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];

            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();

            $valorNegociacao = $this->contratoService->atualizarValoresNegociacaoContrato($nrnegociacaocontrato, $nrorg, $cdoperador);
            $response->addDataSet(new DataSet("retorno", array("valorNegociacao" => $valorNegociacao)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function atualizarValoresNegociacaoAoEditarItemSemComposicao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nritemocontrato  = $row["NRITEMOCONTRATO"];
            $vrtotitemcont    = $row["VRTOTITEMCONT"];
            $localAtendNovo   = $row["NRESTRUTURAM_NEW"];
            $localAtendAntigo = $row["NRESTRUTURAM_OLD"];
            
            $this->contratoService->atualizarValoresNegociacaoAoEditarItemSemComposicao($nritemocontrato, $vrtotitemcont, $localAtendNovo, $localAtendAntigo);
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /* Valida se foi realizado reajuste dos valores do contrato antes de regerar as parcelas de faturamento e/ou alterar a vig\ência do contrato */
    public function validaReajuste(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $dtinicio      = $row["DTINICIO"];
            $dtfinal       = $row["DTFINAL"];
            $nrorg         = $this->environment->getNrOrg();
            
            $reajuste = $this->contratoService->validaReajuste($nrseqcontrato, $dtinicio, $dtfinal, $nrorg);
            
            $response->addDataSet(new DataSet("retorno", array("valida" => $reajuste)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * valida as informações dos parâmetros de faturamento do contrato
    **/
    
    public function validarParametrosFaturamentoCont(Request\Row $request, Response $response) {
        try {
            $row   = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            
            $this->contratoService->validarParametrosFaturamentoCont($nrseqcontrato, $nrorg);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function excluirParcelasNegociacaoContrato(Request\Row $request, Response $response) {
        try {
            $row   = $request->getRow();
            $nrorg = $this->environment->getNrOrg(); 
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            
            $this->contratoService->excluirParcelasNegociacao($nrnegociacaocontrato, $nrorg);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * valida a existencia de descontos e acrescimos no contratos
    **/
    public function validarExistenciaDescontoAcrescimoContrato(Request\Row $request, Response $response) {
        try {
            $row   = $request->getRow();
            $nrorg = $this->environment->getNrOrg(); 
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            
            $descontos  = $this->contratoService->getDescontosContrato($nrseqcontrato, $nrorg);
            $itens      = $this->contratoService->getItensContratos($nrseqcontrato, $nrorg);
            $possuiAlteracaoValor = count($descontos) > 0;
            
            if(!$possuiAlteracaoValor) {
                foreach($itens as $item) {
                    if($item["VRACRESCIMOITEM"] <> 0) {
                        $possuiAlteracaoValor = true;
                        break;
                    }
                }
            }
            
            $response->addDataSet(new DataSet("fct_contrato_widget", array($possuiAlteracaoValor)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * A função "completaPercentual" verifica qual o percentual faltante para inserção na página "índices Financeiros Padrões"
     */
    
    public function completaPercentual(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrorg = $this->environment->getNrOrg();
            $nrseqcontrato = $row['NRSEQCONTRATO'];
    
            $result = $this->contratoService->completaPercentual($nrseqcontrato, $nrorg);
            
            $response->addDataSet(new DataSet("completaPercentual", array("PERCENTUAL" => $result)));

        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
     /**
     * A função "validaPercentual" valida o percentual para inserção na página "índices Financeiros Padrões"
     */
    
    public function validaPercentual(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrorg = $this->environment->getNrOrg(); 
            $nrseqcontrato   = $row['NRSEQCONTRATO'];
            $nrpercentindice = $row['NRPERCENTINDICE'];
            $cdindice        = $row["CDINDICE"];
            
            $result = $this->contratoService->validaPercentual($nrpercentindice, $nrseqcontrato, $nrorg, $cdindice);
            $response->addDataSet(new DataSet("validaPercentual", array("PERCENTUAL" => $result)));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "insereIndicesPadroes" insere os indices padrões que já estiverem preenchidos para o Parceiro de negócio ou para o escopo de faturamento, com prioridade para o parceiro de negócio.
     */
    
    public function insereIndicesPadroes(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrorg = $this->environment->getNrOrg(); 
            $operador = $this->environment->getCdOperador();
            $nrseqcontrato   = $row['NRSEQCONTRATO'];
            $nrtipoorcamento = $row['NRTIPOORCAMENTO']; 
            $nrparcnegocio   = $row['NRPARCNEGOCIO'];
    
            $this->contratoService->insereIndicesPadroes($nrtipoorcamento, $nrparcnegocio, $nrseqcontrato, $nrorg, $operador);
            
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida se há contratos reajustados automaticamente ou contratos vencidos para uma 
     * determinada data
     */
    public function validarExistenciaRenovacaoAutomatica(Request\Row $request, Response $response) {
        try{
            $row      = $request->getRow();
            $nrorg    = $this->environment->getNrOrg(); 

            $dataReferencia      = $row['DTREFERENCIA'];
            $possuiContRenovacao = $this->contratoService->validarExistenciaRenovacaoAutomatica($dataReferencia, $nrorg);
            
            $response->addDataSet(new DataSet("fct_contrato_widget", array("POSSUICONTRATOSVENC" => $possuiContRenovacao)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**

     * realiza o update da proposta para incluir / alterar o campo "DSALTERACONTRATUAIS"
    **/
    public function updateDsAlteracoesContr(Request\Row $request, Response $response) {
        try {
            $row   = $request->getRow();
            $nrseqcontrato      = $row["NRSEQCONTRATO"];
            $dsalteracontratuais = $row["DSALTERACONTRATUAIS"];
            
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $this->contratoService->updateDsAlteracoesContr($nrseqcontrato, $dsalteracontratuais, $nrorg, $cdoperador);
            
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida se o operador tem acesso à página "Alterações Contratuais"
    **/
    public function validaAlteracoesContratuais(Request\Row $request, Response $response) {
        try {
            $nrorg = $this->environment->getNrOrg();

            $resultado = $this->contratoService->validaAlteracoesContratuais($nrorg);
            $response->addDataSet(new DataSet("validaAlteracoesContratuais", array("resultado" => $resultado)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     *recebe os dados de "Alterações Contratuais"
    **/
    public function getAlteracaoContratual(Request\Row $request, Response $response) {
        try {
            $row   = $request->getRow();
            $nrseqcontrato  = $row["NRSEQCONTRATO"];
            
            $nrorg = $this->environment->getNrOrg();

            $resultado = $this->contratoService->getAlteracaoContratual($nrseqcontrato, $nrorg);
            $response->addDataSet(new DataSet("getAlteracaoContratual", array("resultado" => $resultado)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /*
     * insere os signatários padrões de modelo de proposta que foram buscados na parametrização SM 
    **/
    public function inserirAssinaturasContr(Request\Row $request, Response $response){
        try {
            $row      = $request->getRow();
            
            $nrmodelocontratoc = $row["NRSEQMODCONTRA"];
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrorg    = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            
            $this->contratoService->inserirAssinaturasContr($nrmodelocontratoc, $nrseqcontrato, $nrorg, $operador);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "validaAssinaturaCont" realiza a validação das assinaturas para evitar a inclusão repetida de um parceiro.
     */
    public function validaAssinaturaCont(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $result = $this->contratoService->validaAssinaturaCont($row);
            $response->addDataSet(new DataSet("validaAssinaturaCont", array("valida" => $result)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida se o token da certsign está preenchido
     */
    public function validaIdCertsign(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrseqcontrato = isset($row['NRSEQCONTRATO']) ? $row['NRSEQCONTRATO'] : null;
            $nrorg = $this->environment->getNrorg();
            
            $result = $this->contratoService->validaIdCertsign($nrseqcontrato, $nrorg);
            $response->addDataSet(new DataSet("validaIdCertsign", $result));
            
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Envia novamente o e-mail de assinatura para os signatários que ainda não realizaram a assinatura
     */
    public function solicitarAssinatura(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrorg         = $this->environment->getNrorg();
            $operador      = $this->environment->getCdOperador();
            
            $result = $this->contratoService->solicitarAssinatura($nrseqcontrato, $nrorg, $operador);
            $response->addDataSet(new DataSet("solicitarAssinatura", array("resultado" => $result)));
            
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Exclui o signatário do Certsign e altera a situação para 'Assinado Manualmente'.
     */
    public function assinadoManualmente(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            
            $signatarios = $row["SIGNATARIO"];
            $checked     = $row["CHECKED"];
            $nrorg       = $this->environment->getNrorg();
            $operador    = $this->environment->getCdOperador();
            
            $result = $this->contratoService->assinadoManualmente($signatarios, $checked, $nrorg, $operador);
            $response->addDataSet(new DataSet("assinadoManualmente", array("resultado" => $result)));
            
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * Exclui o signatário do Certsign e altera a situação para 'Assinado Manualmente'.
     */
    public function assinadoManualmenteDoc(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            
            $signatarios    = $row["SIGNATARIO"];
            $checked        = $row["CHECKED"];
            $nrorg          = $this->environment->getNrorg();
            $operador       = $this->environment->getCdOperador();
            
            $result = $this->contratoService->assinadoManualmenteDoc($signatarios, $checked, $nrorg, $operador);
            $response->addDataSet(new DataSet("assinadoManualmenteDoc", array("resultado" => $result)));
            
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * valida se todas as assinaturas estão aprovadas para aprovar o contrato.
     */
    public function validaAprovacaoCertsign(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row['NRSEQCONTRATO'];
            $nrorg = $this->environment->getNrorg();
            
            $result = $this->contratoService->validaAprovacaoCertsign($nrseqcontrato, $nrorg);
            $response->addDataSet(new DataSet("validaAprovacaoCertsign", array("resultado" => $result)));
            
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "gerarDocumentoCertsign" realiza o upload e cria o documento no ambiente certsign
     */
    public function gerarDocumentoCertsign(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            
            $nrseqcontrato      = $row['NRSEQCONTRATO'];
            $idsituacaocontrato = $row["IDSITUACAOCONTRATO"];
            $nrcontratofct      = $row["NRCONTRATOFCT"];
            $nrorg              = $this->environment->getNrorg();
            $operador           = $this->environment->getCdOperador();
            
            $caminhoArquivos = $this->documentoService->gerarDocumentoContrato($nrseqcontrato, $nrorg, $idsituacaocontrato, 'P');
            
            foreach ($caminhoArquivos as $arq) {
                $arquivos[] = $arq["DADOSDOCUMENTO"];
            }

            foreach($arquivos as $arquivo){
                
                $nmDocumento = $nrcontratofct.'.pdf';
                $documento   = $arquivo["DSCAMINHOARQ"]; 
                
                $result = $this->contratoService->gerarDocumentoCertsign($nrseqcontrato, $nmDocumento, $documento, $nrorg, $operador);
                $response->addDataSet(new DataSet("gerarDocumentoCertsign", array("resultado" => $result)));
            }
        }  
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "inserirSignatario" realiza a inclusão de um novo signatário.
     */
    
    public function inserirSignatario(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $this->contratoService->inserirSignatario($row);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /**
     * A função "excluirSignatariosCertsign" realiza a exclusão de um signatário no Certsign.
     */
    public function excluirSignatariosCertsign(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrseqcontrato     = $row["NRSEQCONTRATO"];
            $nrassdigitalcontr = $row["NRASSDIGITALCONTR"];
            $idsignatcertsign  = $row["IDSIGNATCERTSIGN"];
            $nrcpfsignatario   = $row["NRCPFSIGNATARIO"];
            $nrorg             = $this->environment->getNrorg();
            $operador          = $this->environment->getCdOperador();
            
            $this->contratoService->excluirSignatariosCertsign($nrseqcontrato, $nrassdigitalcontr, $idsignatcertsign, $nrcpfsignatario, $nrorg, $operador);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     /**
     * realiza o download do documento do ambiente Certsign 
    **/
    public function downloadDocEletrDigital(Request\DataSet $request, Response $response) {
        try{
            $row = $request->getDataSet()->getRows()[0];
            
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrorg         = $this->environment->getNrorg();
            
            $this->contratoService->downloadDocEletrDigital($nrseqcontrato, $nrorg);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * realiza a atualização do Nr. de Parcelas da negociação
    **/
    public function atualizarNrParcelas(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            $nrorg                = $this->environment->getNrorg();

            $this->contratoService->atualizarNrParcelas($nrnegociacaocontrato, $nrorg);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Suspende o contrato para pausar o faturamento e as alterações no contrato.
    **/
    public function suspenderContrato(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrseqcontrato       = $row["NRSEQCONTRATO"];
            $idParcelasAnt       = $row["PARCELASANTERIORES"];
            $dsjustifcasuspensao = $row["DSJUSTIFCASUSPENSAO"];
            $cdopersolsuspensao  = $row["CDOPERSOLSUSPENSAO"];
            $nrorg               = $this->environment->getNrorg();
            $operador            = $this->environment->getCdOperador();
            
            $this->contratoService->suspenderContrato($nrseqcontrato, $idParcelasAnt, $nrorg, $operador, $dsjustifcasuspensao, $cdopersolsuspensao);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Cancela a suspensão do contrato
    **/
    public function cancelarSuspensaoContrato(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrseqcontrato  = $row["NRSEQCONTRATO"];
            $nrorg          = $this->environment->getNrorg();
            $operador       = $this->environment->getCdOperador();
            
            $this->contratoService->cancelarSuspensaoContrato($nrseqcontrato, $nrorg, $operador);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Verifica se um contrato possui alguma negociação feita para um de seus itens ou elementos.
     */
    public function verificarSeContratoPossuiNegociacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato    = $row["NRSEQCONTRATO"];
            $nrseqcontratopai = $row["NRSEQCONTRATOPAI"];
            $nrorg            = $this->environment->getNrorg();
            
            $possuiNegociacao = $this->contratoService->verificarSeContratoPossuiNegociacao($nrseqcontrato, $nrseqcontratopai, $nrorg);
            $response->addDataSet(new DataSet("retorno", array("possuiNegociacao" => $possuiNegociacao)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * verifica se um contrato tem parcelas já faturadas. 
     * Caso positivo, exclui as parcelas e retorna a mensagem falando quaanto ja foi faturado
     */
    public function verificarParcelasContReneg(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrorg         = $this->environment->getNrorg();
            
            $result = $this->contratoService->verificarParcelasContReneg($nrseqcontrato, $nrorg);
            $response->addDataSet(new DataSet("verificarParcelasContReneg", array("resultado" => $result)));

        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
     /* Retorna o parametro validação para a geração de 
     * documentos de encerramento em Docx ou PDF
    **/
    public function getParametroExtensaoDocEncerra(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $nrtiporescisao = $row["NRTIPORESCISAO"];
            $nrorg          = $this->environment->getNrOrg();
            
            $param = $this->contratoService->getParametroExtensaoDocEncerra($nrtiporescisao, $nrorg);
            $response->addDataSet(new DataSet("IDPERMITEDOCX", $param));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Gera documento de rescisão do contrato
    **/
    
    public function gerarDocumentoContratoRescisao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $contratos      = $row["CONTRATOS"];
            $nrparcnegocio  = $row["NRPARCNEGOCIO"];
            $nrSeqContrato  = $row["NRSEQCONTRATO"];
            
            $nrTipoRescisao = $row["NRTIPORESCISAO"];
            $nrorg          = $this->environment->getNrOrg();
            $idSituacao     = 'E';
            $idExtensao     = $row["IDDOCX"];
            
            if ($contratos == null){
                
                $contr = array();
                $contr[$nrparcnegocio][] = $nrSeqContrato;
                
            } 
            else{
                foreach($contratos as $contrato)
                    $parcArray[$contrato["NRPARCNEGOCIO"]][] = $contrato["NRSEQCONTRATO"];
                $contr = $parcArray;
            }
            
            $caminhoArquivos = $this->documentoService->gerarDocumentoContratoRescisao($contr, $nrTipoRescisao, $nrorg, $idSituacao, $idExtensao);
            
            if (count($caminhoArquivos) > 0) {
                $messageerror = null;
                $caminhosFisicos = array();
                foreach ($caminhoArquivos as $caminho) {
                    foreach ($caminho as $chave => $arq) {
                        if(!$arq["CAMINHO"])
                            $messageerror = "Atenção!<br><br>Não é possível gerar o documento deste contrato pois não há itens ligados a ele.";
                            
                        $retorno[] = $arq["CAMINHO"];
                    }
                }
                
                $response->addMessage(new Response\Message($messageerror));
                if (count($retorno) > 0) 
                    $response->addDataSet(new DataSet("documentos", array("links" => $retorno)));
                else 
                    throw new \Exception("Falha ao anexar o(s) documento(s) no FileServer.");
            } else {
                throw new \Exception("Falha ao gerar o(s) documento(s).");
            }
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "aprovarContrMassa" realiza a aprovação de diversos contratos
     */
    public function aprovarContrMassa(Request\Row $request, Response $response) {
        try{
            $contratos = $request->getRow();
            $nrorg     = $this->environment->getNrorg();
            $operador  = $this->environment->getCdOperador();
            
            $result = $this->contratoService->aprovarContrMassa($contratos, $nrorg, $operador);
            if (count($result["ITENS"]) > 0) {// se houve algum item inserido na rotina de geração automática então deve chamar as procedures.
                $result["ITENS"] = $this->finalizacaoDeContratosGeradosAutomaticamente($result["ITENS"][0]);
            }
            
            $response->addDataSet(new DataSet("aprovarContrMassa", array("resultado" => $result)));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * Retorna o parametro validação para a geração de 
     * documentos de encerramento em Docx ou PDF
    **/
    public function encerrarContratoEmMassa(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $dadosrescisao = $row["DADOSRESCISAO"];
            $contratos     = $row["CONTRATOS"];
            $idDocX        = $row["IDDOCX"];
            $nrorg         = $this->environment->getNrOrg();
            $operador      = $this->environment->getCdOperador();
            
            $result = $this->contratoService->encerrarContratoEmMassa($dadosrescisao, $contratos, $idDocX, $nrorg, $operador);
            $response->addDataSet(new DataSet("encerrarContratoEmMassa", array("resultado" => $result)));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida se é necessário filtrar os serviços na inserção de contratos manuais
    **/
    public function validaFiltroServico(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $tipoorcamento = $row["NRTIPOORCAMENTO"];
            $nrorg         = $this->environment->getNrOrg();
            
            $result = $this->contratoService->validaFiltroServico($tipoorcamento, $nrorg);
            $response->addDataSet(new DataSet("validaFiltroServico", array("resultado" => $result)));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * A função "regerarFaturaMassa" regera o faturamento de diversos contratos
     */
    public function regerarFaturaMassa(Request\Row $request, Response $response) {
        try{
            $contratos = $request->getRow();
            $nrorg     = $this->environment->getNrorg();
            $operador  = $this->environment->getCdOperador();
            
            $result = $this->contratoService->regerarFaturaMassa($contratos, $nrorg, $operador);
            
            $response->addDataSet(new DataSet("regerarFaturaMassa", array("resultado" => $result)));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getUtilizaComposicaoServico(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrorg         = $this->environment->getNrorg();
            
            $utilizaComposicao = $this->contratoService->getUtilizaComposicaoServico($nrseqcontrato, $nrorg);
            
            $response->addDataSet(new DataSet("retorno", array("utilizaComposicao" => $utilizaComposicao)));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * A função "insereOperadoresNotifica" realiza a inserção dos operadores do sistema que devem ser notificados.
     */
    public function insereOperadoresNotifica(Request\Row $request, Response $response) {
        try{
            $row = $request->getRow();
            
            $nrassdigitalcontr = $row["NRASSDIGITALCONTR"];
            $operadores = $row["OPERADORES"];
            
            $this->contratoService->insereOperadoresNotifica($nrassdigitalcontr, $operadores);

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
     public function recebeOrdenacaoENotificacoes(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $nrseqmodcontra = $row["NRSEQMODCONTRA"];
            $idtpsignatario = $row["IDTPSIGNATARIO"];
            $nrorg          = $this->environment->getNrOrg();
            
            $result = $this->contratoService->recebeOrdenacaoENotificacoes($nrseqmodcontra, $idtpsignatario, $nrorg);
            $response->addDataSet(new DataSet("recebeOrdenacaoENotificacoes", array("resultado" => $result)));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida a existência de todos os produtos na aba de Faturamento.
     * @returns {DataSet} O DataSet contém um boolean com nome resultado, dizendo se existe algum item de negociação sem produto para faturamento. 
    */
    public function validarProdutoFaturamento(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrseqcontra = $row["NRSEQCONTRATO"];
            
            $result = $this->contratoService->validarProdutoFaturamento($nrseqcontra);
            $response->addDataSet(new DataSet("validarProdutoFaturamento", array("resultado" => $result)));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    /**
     * Procedimento para verificar se este contrato possui dados de reajuste.
     */
    public function possuiDadosReajuste(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow(); 
            $nrseqcontrato       = $row['NRSEQCONTRATO']; 
            $nrorg    = $this->environment->getNrOrg();
            $possui = $this->contratoService->possuiDadosDeReajuste($nrorg, $nrseqcontrato);
            $response->addDataSet(new DataSet("Desfeito", array("Possui" => $possui)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    
    /**
     * Procedimento para verificar se este contrato possui dados de reajuste.
     */
    public function possuiDadosRenovacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow(); 
            $nrseqcontrato       = $row['NRSEQCONTRATO']; 
            $nrorg    = $this->environment->getNrOrg(); 
            $possui = $this->contratoService->possuiDadosDeRenovacao($nrorg,$nrseqcontrato);
            $response->addDataSet(new DataSet("Desfeito", array("Possui" => $possui)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    /**
     * Procedimento responsável por desfazer o reajuste de um contrato.
     */
    public function desfazerReajusteContrato(Request\Row $request, Response $response) {
        try {
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $row = $request->getRow(); 
            $nrseqcontrato  = $row['NRSEQCONTRATO']; 
            $nrseqprorrcont = $row['NRSEQPRORRCONT']; 
            $renovacaoJunto = $row["JUNTO"];
            
            $this->contratoService->desfazerReajusteDoContratoProcessoCompleto($nrseqcontrato, $nrseqprorrcont, $renovacaoJunto, $nrorg, $cdoperador);
            /*$this->contratoService->desfazerReajusteDoContrato($nrorg, $nrseqcontrato, $nrseqprorrcont, $renovacaoJunto);
            
            if ($renovacaoJunto)
                $this->contratoService->refazerIntegracaoOMPeriodoExtendido($nrseqcontrato, $nrorg);
            else
                $this->contratoService->atualizarValorLinhaProduto($nrseqcontrato, $cdoperador, $nrorg);
            
            $this->contratoService->executarProcedureFctGerarParcelasFatura($nrseqcontrato, $nrorg, $cdoperador);*/
            
            $response->addDataSet(new DataSet("Desfeito", array("Desfeito"=>true)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Procedimento responsável por desfazer o reajuste de um contrato.
     */
    public function desfazerRenovacaoContrato(Request\Row $request, Response $response) {
        try {
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $row = $request->getRow(); 
            $nrseqcontrato  = $row['NRSEQCONTRATO'];
            $nrseqprorrcont = $row['NRSEQPRORRCONT'];
            $reajusteJunto  = $row["JUNTO"];
            
            $this->contratoService->desfazerRenovacaoDoContratoProcessoCompleto($nrseqcontrato, $nrseqprorrcont, $reajusteJunto, $nrorg, $cdoperador);
            
            /*$this->contratoService->desfazerRenovacaoDoContrato($nrorg, $nrseqcontrato, $nrseqprorrcont, $reajusteJunto);
            
            $this->contratoService->refazerIntegracaoOMPeriodoExtendido($nrseqcontrato, $nrorg);
            $this->contratoService->executarProcedureFctGerarParcelasFatura($nrseqcontrato, $nrorg, $cdoperador);*/
            
            $response->addDataSet(new DataSet("Desfeito", array("Desfeito" => true)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    } 
    
    public function validaBloqueiaDataInicialContReneg(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $nrseqmodcontra = $row["NRSEQMODCONTRA"];
            $nrorg         = $this->environment->getNrOrg();
            
            $result = $this->contratoService->validaBloqueiaDataInicialContReneg($nrseqmodcontra, $nrorg);
            $response->addDataSet(new DataSet("validaBloqueiaDataInicialContReneg", array("resultado" => $result)));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function salvarCentroDeCusto(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $tipo = $row["TIPO"]; 
            $cdCentCust = $row["CDCENTCUST"]; 
            $nrSeqContrato = $row["NRSEQCONTRATO"]; 
            $nrCompServico = $row["NRCOMPSERVICO"]; 
            $idElemento = $row["IDELEMENTO"];
            $result = $this->contratoService->salvarCentroDeCusto($tipo, $cdCentCust, $nrSeqContrato, $nrCompServico, $idElemento );
            $response->addDataSet(new DataSet("atualizouCentroDeCusto", array("resultado" => $result)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
    public function gerarContratoFaturamento(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrParc = $row["NRPARCNEGOCIO"];
            $nrSeq = $row["NRSEQCONTRATO"];
            $result = $this->contratoService->gerarContratoParaFaturamentoDeServicos($nrParc, $nrSeq);
            //continuacao do service para evitar dependência circular
            $r = $this->finalizacaoDeContratosGeradosAutomaticamente($result);
            $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function removerRescisao(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $nrSeqContrato = $row["NRSEQCONTRATO"]; 
            $result = $this->contratoService->removerRescisaoContrato($cdoperador, $nrorg, $nrSeqContrato);
            $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Função privada dedicada a terminar a rotina de geração de contratode faturamento de serviços de modo automático sem interferir com a arquitetura atual do projeto.
     * Essa função foi feita para evitar o problema de dependência circular.
     * @params $result conjunto de todos os itens inseridos no contrato (select * do FCT_ITEMCONTRATO).
     * @returns  os itens inalterados.
    */
    private function finalizacaoDeContratosGeradosAutomaticamente($result) {
        try{
            if (!empty($result)) {
                $it = $result[0][0];
                $cont = array(
                   "NRSEQCONTRATO" => $it["NRSEQCONTRATO"],
                   "NRORG"         => $it["NRORG"],
                   "CDOPERADOR"    => $it["CDOPERINCLUSAO"]
                );
                foreach($result as $item){
                    $item = $item[0]; // remoção do indice do array
                    $row = array(
                        "NRSEQCONTRATO"     => $item["NRSEQCONTRATO"],
                        "CDSERVORCA"        => $item["CDSERVORCA"],
                        "NRITEMOCONTRATO"   => $item["NRITEMOCONTRATO"],
                        "NRESTRUTURAM"      => $item["NRESTRUTURAM"],
                        "VRTOTITEMCONT"     => $item["VRTOTITEMCONT"],
                        "NRITEMCONTRATOPAI" => $item["NRITEMCONTRATOPAI"]
                    );
                    $this->itemContratoService->inserirItemNegociacaoValoresContrato($row, $item["NRORG"], $item["CDOPERINCLUSAO"], $this->codigoEscopoFaturamentoContratoGerado); 
                }
               $this->contratoService->executarProcedureFctGerarParcelasFatura($cont["NRSEQCONTRATO"], $cont["NRORG"], $cont["CDOPERADOR"]); 
            }
            return $result;
        }
        catch(\Exception $e){
            throw $e;    
        }
    }
    
    public function gerarContratoAutomaticamenteParaFaturamento() {
        try {
            $callback = function($result){
                $this->finalizacaoDeContratosGeradosAutomaticamente($result);
            };
            $this->contratoService->gerarContratosServicosAutomatico($callback);
            echo "Processo realizado com sucesso!";
            // $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        }
        catch(\Exception $e) {
            // $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
            echo "Não foi possível gerar contratos para os parceiros. Erro: " . $e->getMessage() . "\n";
        }
    }
    
    public function inserirSeguroContrato(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $nrorg            = $this->environment->getNrOrg();
            $operador         = ($row["IDVIGENTE"] == "S") ? $this->environment->getCdOperador() : $row["CDOPERINCLUSAO"];
            $nrseqcontrato    = $row["NRSEQCONTRATO"];
            $nrapolice        = $row["NRAPOLICE"];
            $dsmodalidade     = $row["DSMODALIDADE"];
            $dtiniseguro      = $row["DTINISEGURO"];
            $dtfinseguro      = $row["DTFINSEGURO"];
            $vrtaxaseguro     = $row["VRTAXASEGURO"];
            $vrapolice        = $row["VRAPOLICE"];
            $nrparccorretora  = $row["NRPARCCORRETORA"];
            $nrparcseguradora = $row["NRPARCSEGURADORA"];
            $idvigente        = $row["IDVIGENTE"];
            $dtinclusao       = isset($row["DTINCLUSAO"]) ? $row["DTINCLUSAO"] : null;
            
            $result = $this->contratoService->inserirSeguroContrato($nrapolice, $dsmodalidade, $dtiniseguro, $dtfinseguro, $vrtaxaseguro, $vrapolice,$nrparccorretora, $nrparcseguradora, $nrorg, $operador, $idvigente, $nrseqcontrato, $dtinclusao);
            //continuacao do service para evitar dependência circular
            $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function removerRescisaoMultiplos(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $contratos = $row["CONTRATOS"]; 
            $result = $this->contratoService->removerRescisaoContratoMultiplos($cdoperador, $nrorg, $contratos);
            $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function solicitarRescisao(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrseqcontrato       = $row['NRSEQCONTRATO']; 
            $idsituacaocontrato  = $row['IDSITUACAOCONTRATO']; 
            $dtrecisao           = isset($row["DTRECISAOCONTR"])  ? $row['DTRECISAOCONTR'] : null; 
            $idtpexclusaoparcela = $row['IDTPEXCLUSAOPARCELA']; 
            $nrtiporescisao      = $row["NRTIPORESCISAO"];
            $idDocX              = $row["IDDOCX"];
            $dtpedrescisaocontr = $row["DTPEDRESCISAOCONTR"];
            $nrmotivorescisao   = $row["NRMOTIVORESCISAO"];
            $dsjustificaencerra = $row["DSJUSTIFICAENCERRA"];
            $vrpendenciafin     = $row["VRPENDENCIAFIN"];
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $result = $this->contratoService->solicitarRescisaoContrato($dtpedrescisaocontr, $dtrecisao, $nrmotivorescisao, $nrtiporescisao, $vrpendenciafin, $dsjustificaencerra, $nrorg, $cdoperador, $nrseqcontrato, $idtpexclusaoparcela);
            $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function solicitarRescisaoMultiplos(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrseqcontrato       = $row['NRSEQCONTRATO']; 
            $idsituacaocontrato  = $row['IDSITUACAOCONTRATO']; 
            $dtrecisao           = isset($row["DTRECISAOCONTR"]) ? $row['DTRECISAOCONTR'] : null; 
            $idtpexclusaoparcela = isset($row["IDTPEXCLUSAOPARCELA"]) ? $row['IDTPEXCLUSAOPARCELA'] : 2; 
            $nrtiporescisao      = $row["NRTIPORESCISAO"];
            $idDocX              = $row["IDDOCX"];
            $dtpedrescisaocontr = $row["DTPEDRESCISAOCONTR"];
            $nrmotivorescisao   = $row["NRMOTIVORESCISAO"];
            $dsjustificaencerra = $row["DSJUSTIFICAENCERRA"];
            $vrpendenciafin     = $row["VRPENDENCIAFIN"];
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $contratos = $row["CONTRATOS"]; 
            $result = $this->contratoService->solicitarRescisaoContratoMultiplos($dtpedrescisaocontr, $dtrecisao, $nrmotivorescisao, $nrtiporescisao, $vrpendenciafin, $dsjustificaencerra, $nrorg, $cdoperador, $contratos, $idtpexclusaoparcela);
            $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function salvarHistoricoSeguro(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $nrorg = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            $nrsegurocontrato = $row["NRSEGUROCONTRATO"];
            $result = $this->contratoService->salvarHistoricoSeguro($nrorg, $operador, $nrsegurocontrato);
            $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        }catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validarMedicaoServico(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrnegociacaocontrato = $row["NRNEGOCIACAOCONTRATO"];
            $nrtipoorcamento      = $row["NRTIPOORCAMENTO"];
            
            $medicaoServico = $this->contratoService->validarMedicaoServico($nrnegociacaocontrato, $nrtipoorcamento);
            $response->addDataSet(new DataSet("retorno", array("medicaoServico" => $medicaoServico)));
        }
        catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function salvarAlteraSituAssina(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            
            $widassinatura = $row["WIDASSINATURA"];
            $widsituacao = $row["WIDSITUACAO"];
             
            $nrorg = $this->environment->getNrOrg();
            $operador = $this->environment->getCdOperador();
            
            $result = $this->contratoService->salvarAlteraSituAssina($widassinatura, $widsituacao, $nrorg, $operador);
            $response->addDataSet(new DataSet("salvarAlteraSituAssina", array("resultado" => $result)));
            
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validaAlteracaoEscopo(Request\Row $request, Response $response){
        try{
            $row = $request->getRow();
            $altEscopo = $row["ALT_ESCOPO"];
             
            if ($altEscopo == true) {
                $contrato        = $row["CONTRATO"];
                $nrtipoorcamento = $row["NRTIPOORCAMENTO"];
                $nrseqmodcontra  = $row["NRSEQMODCONTRA"];
                
                $nrorg = $this->environment->getNrOrg();
                $operador = $this->environment->getCdOperador();
                
                $result = $this->contratoService->validaAlteracaoEscopo($contrato, $nrtipoorcamento, $nrseqmodcontra, $nrorg, $operador);
                $response->addDataSet(new DataSet("validaAlteracaoEscopo", array("resultado" => $result)));
            }
            else {
                $response->addDataSet(new DataSet("validaAlteracaoEscopo", array("resultado" => "OK"))); 
            }
        }
        catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function inserirDocumentos(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $seqModContrato = $row['NRSEQMODCONTRA'];
            $seqContrato = $row['NRSEQCONTRATO'];
            $nrorg = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            $result = $this->contratoService->inserirDocumentos($seqModContrato, $seqContrato, $nrorg, $cdoperador);
            $response->addDataSet(new DataSet("request", array("resulted" => $result)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getQuantidadeItensMoedaDiferenteContrato(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            $nrseqcontrato = $row["NRSEQCONTRATO"];

            $qtd = $this->contratoService->getQuantidadeItensMoedaDiferenteContrato($nrseqcontrato);
            $response->addDataSet(new DataSet("retorno", ["qtd" => $qtd]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

}

?>