<?php
namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class Sm00006001TabelaPrecoServicoFactory
{
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * A função "validaServicoTabela" verifica se o serviço 
     * que está sendo relacionado já está relacionado na tabela.
     */
    
    public function validaServicoTabela($params){
        
        $sql = "SELECT COUNT(*) AS VALIDASERV
                  FROM FCT_ITPRECBASESERV
                 WHERE CDSERVORCA IN (".$params['CDSERVORCA'].")
                   AND CDTPMEDIDASERV = :CDTPMEDIDASERV
                   AND CDPRECBASESERV = :CDPRECBASESERV
                   AND NRORG = :NRORG";
        
        unset($params['CDSERVORCA']);
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $return = $std->fetchAll();
        return $return;
    }

    function insertTabPrecos(array $params){

        $sql = "INSERT INTO FCT_PRECBASESERV (
            CDPRECBASESERV,         DSPRECBASESERV,         NRMOEDA,
            NRORG,                  CDOPERINCLUSAO,         DTINCLUSAO
        )
        VALUES (
            :CDPRECBASESERV,        :DSPRECBASESERV,        :NRMOEDA,
            :NRORG,                 :CDOPERINCLUSAO,        SYSDATE
        )";

        $this->connection->executeStatement($sql, $params);
    }

    function insertItPrecBaseServ(array $params){

        $sql = "INSERT INTO FCT_ITPRECBASESERV (
            NRSEQITPRECBSERV,       CDPRECBASESERV,         DTINIPBSERV,
            CDTPMEDIDASERV,         CDSERVORCA,             VRPRECO,
            DTFIMVALPBSERV,         CDFILIAL,               NRORG,
            DTINCLUSAO,             DTULTATU,               NRORGINCLUSAO,
            CDOPERINCLUSAO,         NRORGULTATU,            CDOPERULTATU,
            IDATIVO            
        )
        VALUES (
            :NRSEQITPRECBSERV,       :CDPRECBASESERV,         :DTINIPBSERV,
            :CDTPMEDIDASERV,         :CDSERVORCA,             :VRPRECO,
            :DTFIMVALPBSERV,         :CDFILIAL,               :NRORG,
            :DTINCLUSAO,             :DTULTATU,               :NRORGINCLUSAO,
            :CDOPERINCLUSAO,         :NRORGULTATU,            :CDOPERULTATU,
            :IDATIVO           
        )";

        $this->connection->executeStatement($sql, $params);

    }

    function insertVariacao(array $params){

        $sql = "INSERT INTO FCT_VARIACAOPRECBASESERV (
            NRVARIACAOPRECBASESERV, NRSEQITPRECBSERV,       CDTPMEDIDASERV,
            DTINIPBSERV,            QTMEDIDAINIVARIACAO,    QTMEDIDAFIMVARIACAO,
            VRPRECOVARIACAO,        CDTPMEDIDAVARIAVEL,     IDMULTIPVARIACAO,
            NRORG,                  IDATIVO,                DTINCLUSAO,
            DTULTATU,               QTFRANQUIA,             NRORGINCLUSAO,
            CDOPERINCLUSAO,         NRORGULTATU,            CDOPERULTATU
        )
        VALUES (
            :NRVARIACAOPRECBASESERV, :NRSEQITPRECBSERV,       :CDTPMEDIDASERV,
            :DTINIPBSERV,            :QTMEDIDAINIVARIACAO,    :QTMEDIDAFIMVARIACAO,
            :VRPRECOVARIACAO,        :CDTPMEDIDAVARIAVEL,     :IDMULTIPVARIACAO,
            :NRORG,                  :IDATIVO,                :DTINCLUSAO,
            :DTULTATU,               :QTFRANQUIA,             :NRORGINCLUSAO,
            :CDOPERINCLUSAO,         :NRORGULTATU,            :CDOPERULTATU          
        )";

        $this->connection->executeStatement($sql, $params);
    }

    function getDadosServico($params){

        $sql = "SELECT FCT_ITPRECBASESERV.NRSEQITPRECBSERV, FCT_ITPRECBASESERV.CDPRECBASESERV, TO_CHAR(FCT_ITPRECBASESERV.DTINIPBSERV, 'DD/MM/YYYY') AS DTINIPBSERV, 
        FCT_ITPRECBASESERV.CDTPMEDIDASERV,FCT_ITPRECBASESERV.CDSERVORCA, FCT_ITPRECBASESERV.VRPRECO, TO_CHAR(FCT_ITPRECBASESERV.DTFIMVALPBSERV, 'DD/MM/YYYY') AS DTFIMVALPBSERV, 
        FCT_ITPRECBASESERV.CDFILIAL, FCT_ITPRECBASESERV.NRORG, FCT_ITPRECBASESERV.DTINCLUSAO, FCT_ITPRECBASESERV.DTULTATU, 
        FCT_ITPRECBASESERV.NRORGINCLUSAO, FCT_ITPRECBASESERV.CDOPERINCLUSAO, FCT_ITPRECBASESERV.NRORGULTATU, FCT_ITPRECBASESERV.CDOPERULTATU,FCT_ITPRECBASESERV.IDATIVO 
            FROM FCT_ITPRECBASESERV 
            WHERE FCT_ITPRECBASESERV.CDPRECBASESERV = :CDPRECBASESERV";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $return = $std->fetchAll();
        return $return;
    }

    function getDadosVariacao($params){

        $sql = "SELECT FCT_VARIACAOPRECBASESERV.NRVARIACAOPRECBASESERV, FCT_VARIACAOPRECBASESERV.NRSEQITPRECBSERV, FCT_VARIACAOPRECBASESERV.CDTPMEDIDASERV, MEDIDAFIXA.DSTPMEDIDASERV AS NMMEDIDAFIXA, 
        TO_CHAR(FCT_VARIACAOPRECBASESERV.DTINIPBSERV, 'DD/MM/YYYY') AS DTINIPBSERV, FCT_VARIACAOPRECBASESERV.QTMEDIDAINIVARIACAO, FCT_VARIACAOPRECBASESERV.QTMEDIDAFIMVARIACAO,
        FCT_VARIACAOPRECBASESERV.VRPRECOVARIACAO, FCT_VARIACAOPRECBASESERV.CDTPMEDIDAVARIAVEL, MEDIDAVAR.DSTPMEDIDASERV AS NMMEDIDAVAR, FCT_VARIACAOPRECBASESERV.IDMULTIPVARIACAO, 
        FCT_VARIACAOPRECBASESERV.NRORG, FCT_VARIACAOPRECBASESERV.IDATIVO, FCT_VARIACAOPRECBASESERV.DTINCLUSAO, FCT_VARIACAOPRECBASESERV.DTULTATU, FCT_VARIACAOPRECBASESERV.QTFRANQUIA,
        FCT_VARIACAOPRECBASESERV.NRORGINCLUSAO, FCT_VARIACAOPRECBASESERV.CDOPERINCLUSAO, FCT_VARIACAOPRECBASESERV.NRORGULTATU, FCT_VARIACAOPRECBASESERV.CDOPERULTATU
            FROM FCT_VARIACAOPRECBASESERV 
            JOIN FCT_ITPRECBASESERV ON FCT_ITPRECBASESERV.NRSEQITPRECBSERV = FCT_VARIACAOPRECBASESERV.NRSEQITPRECBSERV 
            LEFT JOIN FCT_TIPOMEDIDASERV MEDIDAFIXA ON MEDIDAFIXA.CDTPMEDIDASERV = FCT_VARIACAOPRECBASESERV.CDTPMEDIDASERV 
            LEFT JOIN FCT_TIPOMEDIDASERV MEDIDAVAR ON MEDIDAVAR.CDTPMEDIDASERV = FCT_VARIACAOPRECBASESERV.CDTPMEDIDAVARIAVEL
            WHERE FCT_VARIACAOPRECBASESERV.NRSEQITPRECBSERV = :NRSEQITPRECBSERV";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        $return = $std->fetchAll();
        return $return;    

    }
    
}
?>